use std::fmt::Debug;

use galactica_content::Content;
use galactica_system::{data::ShipState, phys::PhysSimShipHandle};
use glyphon::TextArea;
use log::info;

use super::scenes::{FlyingScene, LandedScene, OutfitterScene};
use crate::{RenderInput, RenderState};

/// Output from a ui scene step
pub struct UiSceneStepResult {
	/// If Some, switch to this scene
	pub new_scene: Option<UiScenes>,
}

pub trait UiScene<'this>
where
	Self: 'this,
{
	/// Draw this scene
	fn draw(&mut self, input: &RenderInput, state: &mut RenderState);

	/// Update this scene's state for this frame.
	/// Handles clicks, keys, etc.
	fn step(&mut self, input: &RenderInput, state: &mut RenderState) -> UiSceneStepResult;

	fn get_textareas(
		&'this self,
		v: &mut Vec<TextArea<'this>>,
		input: &RenderInput,
		state: &RenderState,
	);
}

pub(super) enum UiScenes {
	Landed(LandedScene),
	Flying(FlyingScene),
	Outfitter(OutfitterScene),
}

impl Debug for UiScenes {
	fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
		match self {
			Self::Flying(_) => write!(f, "UiScenes::Flying"),
			Self::Landed(_) => write!(f, "UiScenes::Landed"),
			Self::Outfitter(_) => write!(f, "UiScenes::Outfitter"),
		}
	}
}

impl UiScenes {
	fn is_flying(&self) -> bool {
		match self {
			Self::Flying(_) => true,
			_ => false,
		}
	}

	fn is_landed(&self) -> bool {
		match self {
			Self::Landed(_) => true,
			_ => false,
		}
	}

	fn is_outfitter(&self) -> bool {
		match self {
			Self::Outfitter(_) => true,
			_ => false,
		}
	}
}

impl<'a> UiScene<'a> for UiScenes {
	fn draw(&mut self, input: &RenderInput, state: &mut RenderState) {
		match self {
			Self::Flying(s) => s.draw(input, state),
			Self::Landed(s) => s.draw(input, state),
			Self::Outfitter(s) => s.draw(input, state),
		}
	}

	fn step(&mut self, input: &RenderInput, state: &mut RenderState) -> UiSceneStepResult {
		match self {
			Self::Flying(s) => s.step(input, state),
			Self::Landed(s) => s.step(input, state),
			Self::Outfitter(s) => s.step(input, state),
		}
	}

	fn get_textareas(
		&'a self,
		v: &mut Vec<TextArea<'a>>,
		input: &RenderInput,
		state: &RenderState,
	) {
		match self {
			Self::Flying(s) => s.get_textareas(v, input, state),
			Self::Landed(s) => s.get_textareas(v, input, state),
			Self::Outfitter(s) => s.get_textareas(v, input, state),
		}
	}
}

pub struct UiManager {
	current_scene: UiScenes,
}

impl UiManager {
	pub fn new(ct: &Content, state: &mut RenderState) -> Self {
		Self {
			current_scene: UiScenes::Flying(FlyingScene::new(ct, state)),
		}
	}

	// TODO: remove this.
	pub fn update_state(&mut self, input: &RenderInput, state: &mut RenderState) {
		let ship_handle = input.player.ship.unwrap();
		let ship = &input
			.phys_img
			.get_ship(&PhysSimShipHandle(ship_handle))
			.unwrap()
			.ship;

		match ship.get_data().get_state() {
			ShipState::Collapsing
			| ShipState::Dead
			| ShipState::Flying { .. }
			| ShipState::Landing { .. }
			| ShipState::UnLanding { .. } => {
				if !self.current_scene.is_flying() {
					self.current_scene = UiScenes::Flying(FlyingScene::new(input.ct, state));
				}
			}

			ShipState::Landed { .. } => {
				if !self.current_scene.is_landed() && !self.current_scene.is_outfitter() {
					self.current_scene = UiScenes::Landed(LandedScene::new(input.ct, state))
				}
			}
		}
	}

	/// Draw all ui elements
	pub fn draw(&mut self, input: &RenderInput, state: &mut RenderState) {
		loop {
			let r = self.current_scene.step(input, state);
			if let Some(new_state) = r.new_scene {
				info!("switching to {:?}", new_state);
				self.current_scene = new_state;
			} else {
				break;
			}
		}

		self.current_scene.draw(input, state);
	}

	/// Textareas to show while player is flying
	pub fn get_textareas(&self, input: &RenderInput, state: &RenderState) -> Vec<TextArea> {
		let mut v = Vec::with_capacity(5);
		self.current_scene.get_textareas(&mut v, input, state);
		return v;
	}
}
