use galactica_content::{Content, SystemObject, SystemObjectHandle};
use galactica_system::{data::ShipState, phys::PhysSimShipHandle};
use glyphon::{cosmic_text::Align, Attrs, Color, Metrics, TextArea, Weight};
use nalgebra::{Point2, Vector2};

use crate::{
	ui::{
		manager::{UiScene, UiSceneStepResult, UiScenes},
		util::{SpriteRect, UiSprite, UiTextArea},
	},
	RenderInput, RenderState,
};

use super::LandedScene;

pub struct OutfitterScene {
	// UI elements
	description: UiTextArea,
	landscape: UiSprite,
	button: UiSprite,

	/// What object we're displaying currently.
	/// Whenever this changes, we need to reflow text.
	current_object: Option<SystemObjectHandle>,

	/// True if we've caught a left click event.
	/// Used for edge detection.
	leftclick_down: bool,
}

impl OutfitterScene {
	pub fn new(ct: &Content, state: &mut RenderState) -> Self {
		let button = UiSprite::new(
			ct,
			ct.get_sprite_handle("ui::button"),
			None,
			SpriteRect {
				pos: Point2::new(0.0, 0.0),
				dim: Vector2::new(200.0, 200.0),
			},
			None,
			None,
		);

		let landscape = UiSprite::from_with_sprite(
			ct,
			&ct.get_ui().landed_landscape,
			ct.get_sprite_handle("ui::landscape::test"),
		);

		let s = Self {
			// height of element in logical pixels
			current_object: None,
			leftclick_down: false,

			description: UiTextArea::new(
				ct,
				state,
				button.get_sprite(),
				Point2::new(0.0, 0.0),
				800.0,
				SpriteRect {
					pos: Point2::new(25.831, 284.883) / 512.0,
					dim: Vector2::new(433.140, 97.220) / 512.0,
				},
				Metrics::new(16.0, 18.0),
				Color::rgb(255, 255, 255),
				Align::Left,
			),

			landscape,
			button,
		};

		return s;
	}

	fn reflow(&mut self, planet: &SystemObject, state: &mut RenderState) {
		self.description.set_text(
			state,
			&planet.desc,
			Attrs::new()
				.weight(Weight::NORMAL)
				.family(glyphon::Family::SansSerif),
		);

		self.current_object = Some(planet.handle);
	}
}

impl<'this> UiScene<'this> for OutfitterScene {
	fn step(&mut self, input: &RenderInput, state: &mut RenderState) -> UiSceneStepResult {
		self.button.step(input, state, None);
		self.landscape.step(input, state, None);

		let mut new_scene = None;
		if input.player.input.pressed_leftclick() && !self.leftclick_down {
			self.leftclick_down = true;
			if self.button.contains_mouse(input, state, None) {
				new_scene = Some(UiScenes::Landed(LandedScene::new(input.ct, state)));
			}
		} else if !input.player.input.pressed_leftclick() {
			self.leftclick_down = false;
		}

		return UiSceneStepResult { new_scene };
	}

	fn draw(&mut self, input: &RenderInput, state: &mut RenderState) {
		// Get required data
		let ship_handle = input.player.ship.unwrap();
		let ship_data = &input
			.phys_img
			.get_ship(&PhysSimShipHandle(ship_handle))
			.unwrap()
			.ship;
		let planet_handle = match ship_data.get_data().get_state() {
			ShipState::Landed { target } => *target,
			_ => unreachable!("tried to draw planet interface while not landed!"),
		};
		let planet = input.ct.get_system_object(planet_handle);

		// Reconfigure for new planet if necessary
		if self
			.current_object
			.map(|x| x != planet_handle)
			.unwrap_or(true)
		{
			self.reflow(planet, state);
		}

		// Draw elements
		self.button.push_to_buffer(input, state, None);
		self.landscape.push_to_buffer(input, state, None);
	}

	fn get_textareas(
		&'this self,
		v: &mut Vec<TextArea<'this>>,
		input: &RenderInput,
		state: &RenderState,
	) {
		v.push(self.description.get_textarea(input, state));
	}
}
